 /* Smarthome Module to check for BLE smart tags for Presence Detection.
 *  Requires Adafruit Feather M0 Wifi (ATWINC1500)
 *  Bluetooth HM-10 (i used the Sunfounder HM-10)
 *  Optionally the Feather Wing OLED Display (128x32). If not used comment out all lines with "Display" or "Button".
 *  BLE Smart Tags (Gigaset G-Tags are good quality and the battery can be changed).
 *  1.Connect HM 10 -"VCC"-pin to the "USB"-pin (5V) of the Feather M0 
 *                -"GND"-pin to the "GND"-pin of the Feather M0
 *                -"TX"-pin to the "RX"-pin of the Feather M0
 *                -"RX"-pin to the "TX"-pin of the Feather M0
 *  2.Install all libraries: Adafruit_SleepyDog.h; Adafruit_SleepyDog.h; Adafruit_WINC1500Udp.h; Adafruit_GFX.h; Adafruit_SSD1306.h (OLED Display;
 *  3.Buffer in file RingBuffer.h of the Adafruit Feather Wifi Library must be set to 128 bytes or higher (The Feather M0 will not work with SoftSerial!!). RingBuffer.h is a hidden file (enable hidden files in your file browser and search for RingBuffer.h)
 *  4.Fill in the SSID and password and UDP inbound port of your network, as well as the IP and Inbound Port of the Loxone Miniserver in the Wifi Setup part.
 *  5.House Status on the display switches by UDP command of the Miniserver. Command "111" means that the House is offline, "222" means that the house is online.
 *  6.UDP command of "000" starts the scan for BLE Devices.
 *  7.!!!!For the Connection Test the Feather M0 sends a UDP message ""testConnection", answer by the Loxone miniserver should be "333" or the feather will reboot!!!!
 *  
 *  Included Adafruit_SleepyDog Library as watchdog to prevent system lockup, countdown is automatically set to max time (normally around 16 seconds).
 *  "If(Serial)" required before "Serial.print" to prevent Serial/USB lockup ;-)
 *  With Feather Wing OLED Display installed "Button A" starts Scan, "Button B" shows Network connectivity, "Button C" switches display on off.
 *  
 *  Power Requirement with Display normally around 60mA, 150mA during short bursts for UDP sending. Wifi Module is programmed for self controlled power management.
 *  
 *  HAVE FUN!!!
 *  
 *  by V.Tögel (V-Labs) 2016 for Adafruit Feather ATWINC 1500 WIFI 
 */
 
#include <Adafruit_SleepyDog.h>
#include <Adafruit_WINC1500.h>
#include <Adafruit_WINC1500Udp.h>
#include <Adafruit_GFX.h>
#include <gfxfont.h>
#include <Adafruit_SSD1306.h>
#include <SPI.h>
///////////////////////////////////////////////////////////////////////////////////////////////////
/*Pin-Definition*/
#define OLED_RESET 4
Adafruit_SSD1306 display(OLED_RESET);

const unsigned char mylogo [] PROGMEM = {
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x70, 0x07, 0x00, 0x01, 0x00, 0x00, 0x80, 
0x02, 0x00, 0x00, 0x40, 0x02, 0x00, 0x00, 0x00, 0x00, 0xf0, 0x0f, 0x20, 0x04, 0xf4, 0x0f, 0xa0, 
0x05, 0x3c, 0x3c, 0x00, 0x04, 0x0c, 0x30, 0x00, 0x04, 0x04, 0x20, 0x00, 0x04, 0x3c, 0x1d, 0x00, 
0x07, 0xfc, 0x3e, 0xa0, 0x06, 0x30, 0x4e, 0x40, 0x04, 0x02, 0x40, 0x00, 0x04, 0x02, 0x40, 0x00, 
0x04, 0x06, 0x40, 0x00, 0x04, 0x12, 0x58, 0x60, 0x01, 0x81, 0xa1, 0xa0, 0x01, 0x43, 0xc2, 0x80, 
0x02, 0xef, 0x7f, 0x00, 0x00, 0x7e, 0x7d, 0x40, 0x00, 0x60, 0x06, 0x40, 0x01, 0x20, 0x00, 0x00, 
0x00, 0x81, 0x80, 0x80, 0x00, 0x41, 0x89, 0x00, 0x00, 0x21, 0x82, 0x00, 0x00, 0x11, 0x80, 0x00, 
0x00, 0x09, 0x90, 0x00, 0x00, 0x04, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 

};

const unsigned char mylogo2 [] PROGMEM = {
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x04, 0x00, 
0x00, 0x80, 0x04, 0x00, 0x00, 0x7f, 0xfc, 0x00, 0x00, 0x7c, 0x1c, 0x00, 0x00, 0x64, 0x1e, 0x00, 
0x00, 0xb0, 0x13, 0x00, 0x01, 0x90, 0x13, 0x80, 0x03, 0x10, 0x31, 0x80, 0x02, 0x18, 0x20, 0xc0, 
0x02, 0x08, 0x60, 0xc0, 0x02, 0x08, 0x40, 0xc0, 0x06, 0x08, 0x40, 0xe0, 0x02, 0x04, 0x40, 0xe0, 
0x02, 0x04, 0xc0, 0xc0, 0x03, 0x04, 0x80, 0xc0, 0x03, 0x85, 0x81, 0xc0, 0x01, 0x87, 0x81, 0x80, 
0x00, 0xc7, 0x03, 0x00, 0x00, 0x63, 0x07, 0x00, 0x00, 0x3b, 0x1e, 0x00, 0x00, 0x1f, 0xfc, 0x00, 
0x00, 0x07, 0xf4, 0x00, 0x00, 0x03, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 
0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 

};

#if (SSD1306_LCDHEIGHT != 32)
#error("Height incorrect, please fix Adafruit_SSD1306.h!");
#endif

// Define the WINC1500 board connections below.
// If you're following the Adafruit WINC1500 board
// guide you don't need to modify these:
#define WINC_CS   8
#define WINC_IRQ  7
#define WINC_RST  4
#define WINC_EN   2     // or, tie EN to VCC and comment this out
// The SPI pins of the WINC1500 (SCK, MOSI, MISO) should be
// connected to the hardware SPI port of the Arduino.
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
/*WiFi Setup*/
// Setup the WINC1500 connection with the pins above and the default hardware SPI.
Adafruit_WINC1500 WiFi(WINC_CS, WINC_IRQ, WINC_RST);

int status = WL_IDLE_STATUS;
char ssid[] = "XXXXXX";                // your network SSID (name)
char pass[] = "XXXXXXXX";            // your network password
int keyIndex = 0;                       // your network key Index number (needed only for WEP)
unsigned int localPort = XXXX;          // local port to listen on (UDP Port of Adafruit Feather M0)

IPAddress remote_ip(XXX, XXX, X, XXX);   //IP Adress of Loxone Miniserver (Smart Home Server)
unsigned int remote_port = XXXX;        // UDP port of Loxone Miniserver (Smart Home Server)

IPAddress feather_ip(XXX, XXX, X, XXX); //Static IP Adress of Feather M0 (comment out if DHCP desired)
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
/*Buffer Definition*/
char packetBuffer[256]; //buffer to hold incoming packet
const int BUFF_SIZE = SERIAL_BUFFER_SIZE;
char buf[BUFF_SIZE];
char bat[BUFF_SIZE];
#define bluetooth Serial1
#ifndef SERIAL_BUFFER_SIZE
  #define SERIAL_BUFFER_SIZE 64 //only needed for feather WICED as seems not to be defined
#endif
///////////////////////////////////////////////////////////////////////////////////////////////////
/*Definition of global variables*/
String blread;
int houseStatus = 1;
int displayOnOff = 1;
int connTest = 0;
int checkTestUdp = 0;
///////////////////////////////////////////////////////////////////////////////////////////////////
/*Function Definitions*/

//Wifi Status
Adafruit_WINC1500UDP Udp;

void printWifiStatus() {
  // print the SSID of the network you're attached to:
  if (Serial) {
    Serial.print("SSID: ");
  Serial.println(WiFi.SSID());}

  // print your WiFi shield's IP address:
  IPAddress ip = WiFi.localIP();
  if (Serial) {Serial.print("IP Address: ");
  Serial.println(ip);}

  // print the received signal strength:
  long rssi = WiFi.RSSI();
  if (Serial) {Serial.print("signal strength (RSSI):");
  Serial.print(rssi);
  Serial.println(" dBm");}
}

// clear serial buffers
void clearAll() {
  // clears Serial
  for (int i = 0; i < bluetooth.available(); i++) {
    bluetooth.read();
  }
  for (int i = 0; i < Serial.available(); i++) {
    Serial.read();
  }
  // clears paket puffers
  for (int i = 0; i < 256; i++) packetBuffer[i] = 0;
  for (int i = 0; i < BUFF_SIZE; i++) buf[i] = 0;
}

// checks for UDP commands
void checkUDP()
{
  int packetSize = Udp.parsePacket();
  if (packetSize)
  {if (Serial) {
    Serial.print("Received packet of size ");
    Serial.println(packetSize);
    Serial.print("From ");
    IPAddress remoteIp = Udp.remoteIP();
    Serial.print(remoteIp);
    Serial.print(", port ");
    Serial.println(Udp.remotePort());}

    // read the packet into packetBuffer
    int len = Udp.read(packetBuffer, 256);
    if (len > 0) packetBuffer[len] = 0;
    if (Serial) {Serial.println("Contents:");
    Serial.println(packetBuffer);}
  }
  delay(10);
}

// sends UDP commands
void sendUDP(String text)
{
  Udp.beginPacket(remote_ip, remote_port);
  Udp.print(text);
  Udp.endPacket();
  delay(100);
}


// scans for BLE devices
void scanbt()
{
  sendUDP("Scan aktiv");
  if (Serial) {Serial.println("//Suche BT-Geraete//");}
  if (displayOnOff == 1) {
  display.clearDisplay();
  display.setCursor(0,0);
  display.println("//Suche BT-Geraete//");
  display.display();}
  bluetooth.print("AT+DISC?");                        // Look for BLE-Devices
  delay(5000);
  bluetooth.readBytes(buf, SERIAL_BUFFER_SIZE);
  if (buf > 0)
  {
    String blread = String(buf);
    
    // clears unneccesary characters
    blread.replace("OK+DISCSOK+DIS0:", "");
    blread.replace("OK+DIS0", "");
    blread.replace("OK+DISCE", "");
    blread.replace("OK+Set","");
    blread.replace("OK+DISC","");
    blread.replace("OK+DI","");
    blread.replace("SS1","");
    blread.replace("S1","");
   
    
    String blread_clean = blread;
    
    // if no BLE device is found
    if (blread_clean == "OK+DISCS")
    {
      if (Serial) {Serial.println("kein Bluetooth Geraet gefunden");}
      sendUDP("kein BT Device");
      delay(2000);
      if (displayOnOff == 1) {
      display.clearDisplay();
      display.setCursor(0,0);
      display.println("kein Bluetooth Geraet \ngefunden");
      display.display();}
    } else {
      if (Serial) {Serial.println(blread_clean);}
     if (displayOnOff == 1) {
      display.clearDisplay();
      display.setCursor(0,0);
      display.print("Gefundene BT-Geraete:");
      display.println(blread_clean);
      display.display();}
      sendUDP(blread_clean);
      delay(2000);
    }
  } else
  {
    sendUDP("Fehler... Keine Daten vom BT-Modul");
    delay(2000);
     if (displayOnOff == 1) {
      display.clearDisplay();
      display.setCursor(0,0);
      display.println("Fehler... Keine Daten \nvom BT-Modul");
      display.display();}
  }
  clearAll();
}

// show and animate logo
void drawlogo() {
display.drawBitmap(5, 0, mylogo, 32, 32, WHITE);
display.setTextSize(1);
display.setTextColor(WHITE);
display.setCursor(45,20);
display.println("-Labs");
display.drawBitmap(90, 0, mylogo2, 32, 32, WHITE);
display.display();
delay(3000);}

//check button A
void showNetwork() {
    if (displayOnOff == 1) {
    display.clearDisplay();
    display.setCursor(0,0);
    display.print("Netzwerk:");
    display.println(WiFi.SSID());
    display.print("Empfang in dBm:");
    display.println(WiFi.RSSI());
     IPAddress ip = WiFi.localIP();
    display.print("IP: ");
    display.println(ip);
    display.display();
    for (int i = 0; i < 3; i++) {
    delay(1000);}}
}

// Online Status of House
void showStatus() {
  if (displayOnOff == 1) {
  if (houseStatus == 1) {
    display.clearDisplay();
    display.setCursor(0,0);
    display.println("Bewohner abwesend!");
    display.println("Haus ist offline!");
    display.println("Alarmanlage aktiv!");
    display.display();}
  }
  if (displayOnOff == 1) {
  if (houseStatus == 2) {
    display.clearDisplay();
    display.setCursor(0,0);
    display.println("Bewohner anwesend!");
    display.println("Haus ist online!");
    display.println("Alarmanlage inaktiv!");
    display.display();}
  }
}

//disable Display
void disableDisplay() {
  if (displayOnOff == 1) {
      displayOnOff = 0;
      display.clearDisplay();
      display.setCursor(0,0);
      display.println("Display aus");
      display.display();
      delay(3000);
      display.clearDisplay();
      display.display();}
    else if (displayOnOff == 0) {
             displayOnOff = 1;
             display.clearDisplay();
             display.setCursor(0,0);
             display.println("Display ein");
             display.display();
             delay(3000);}
}

void connectionTest() { //necessary for Watchdog
  // check if re-connect to Wifi network necessary:
     connTest = (connTest+1);
     if (connTest == 250) {
      sendUDP("testConnection");
      checkTestUdp = 1;
      connTest = 0;
   }
     delay(100);
}

void evaluateConnection() {
  if (checkTestUdp == 0) {
    Watchdog.reset();
  }
  if (checkTestUdp == 1) {
      if (strcmp(packetBuffer, "333") != 0) {
      }
      if (!strcmp(packetBuffer, "333")) {
      checkTestUdp = 0;
      Watchdog.reset(); 
      }
    }
 }
///////////////////////////////////////////////////////////////////////////////////////////////////

void setup() {

#ifdef WINC_EN
  pinMode(WINC_EN, OUTPUT);
  digitalWrite(WINC_EN, HIGH);
#endif

#define BUTTON_A  9
#define BUTTON_B  6
#define BUTTON_C  5
#define LED       13
pinMode(BUTTON_A, INPUT_PULLUP);
pinMode(BUTTON_B, INPUT_PULLUP);
pinMode(BUTTON_C, INPUT_PULLUP);


  Serial.begin(115200);
  delay(500);

  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);  // initialize with the I2C addr 0x3C (for the 128x32)
  // init done
  
  display.clearDisplay();
  display.display();

  // draw logo
  drawlogo();

  // Clear the buffer.
  display.clearDisplay();
  display.setTextSize(1);
    display.setTextColor(WHITE);
    display.setCursor(0,0);

  // check for the presence of the Atwinc1500:
  if (WiFi.status() == WL_NO_SHIELD) {
    if (Serial) {Serial.println("WiFi shield not present");}
    // don't continue:
    while (true);
  }

  WiFi.config(feather_ip);        //comment out if DHCP desired
  
  if (Serial) {Serial.print(status);}
  // attempt to connect to Wifi network:
  while ( status != WL_CONNECTED) {
    if (Serial) {Serial.println("Verbinde zu WLAN-SSID");
    Serial.println(ssid);}
    display.println("Verbinde zu WLAN-SSID");
    display.println(ssid);
    display.display();
    // Connect to WPA/WPA2 network. Change this line if using open or WEP network:
    status = WiFi.begin(ssid, pass);

    // wait 10 seconds for connection:
    delay(10000);
  }
  if (Serial) {Serial.println("Verbunden mit Wlan!");}
  display.clearDisplay();
  display.setCursor(0,0);
  display.println("Verbunden mit Wlan!");
  display.display();
  printWifiStatus();
  delay(3000);
  
  if (Serial) {Serial.println("Starte Verbindung zum Server...");} // if you get a connection, report back via serial
  display.clearDisplay();
  display.setCursor(0,0);
  display.println("Starte Verbindung zum Server...");
  display.display();
  delay(3000);

  WiFi.setSleepMode(M2M_PS_H_AUTOMATIC, 1); // go into power save mode when possible!
  
  Udp.begin(localPort);
  
   // BT-Module Setup
  bluetooth.begin(115200);
  bluetooth.print("AT"); // check if Bluetooth Module works; Answer required "ok"
  if (bluetooth.find("OK")) {
    sendUDP("Anwesenheit BT-Modul ...OK");
   if (Serial) { Serial.println("Anwesenheit BT-Modul ...OK");}
  display.clearDisplay();
  display.setCursor(0,0);
  display.println("Anwesenheit BT-Modul ...OK");
  display.display();
  } else {
    sendUDP("Anwesenheit BT-Modul ...FEHLER");
   if (Serial) {Serial.println("Anwesenheit BT-Modul ...FEHLER");}
   display.clearDisplay();
   display.setCursor(0,0);
   display.println("Anwesenheit BT-Modul ...FEHLER");
   display.display();
  }

  
  bluetooth.print("AT+ROLE1"); // set Bluetooth to central role
  bluetooth.print("AT+RESET"); // Reboot ..required after re-setting ROLE
  delay(1000);
  bluetooth.print("AT+SHOW0"); // shows only the MAC of BLE device
  delay(1000);
  bluetooth.print("AT+IMME1"); // only responds to AT command
  delay(1000);
  
  clearAll();   //Serial dump

  if (Serial) {Serial.println("Warte auf UDP-Befehl");}
  display.clearDisplay();
  display.setCursor(0,0);
  display.println("Warte ab jetzt auf \nUDP Befehl");
  display.display();
  delay(3000);



int countdownMS = Watchdog.enable();
if (Serial) { Serial.print("Enabled the watchdog with max countdown of ");
Serial.print(countdownMS, DEC);
Serial.println(" milliseconds!");
Serial.println();}
  
}

///////////////////////////////////////////////////////////////////////////////////////////////////
void loop() {
  connectionTest();
  checkUDP();

  if (!strcmp(packetBuffer, "000"))
  {
    scanbt();
  }
  if (!strcmp(packetBuffer, "111"))
  { houseStatus = 1;
  }
 
  if (!strcmp(packetBuffer, "222"))
  { houseStatus = 2;
  }

  evaluateConnection(); 


  if(!digitalRead(BUTTON_A)) {
    scanbt();
  }
 
  if(!digitalRead(BUTTON_B)) {
    showNetwork();
  }

  if(!digitalRead(BUTTON_C)) {
    disableDisplay();
  }
  
  showStatus();
}

